package java.security;

import java.security.cert.X509Certificate;
import java.io.ByteArrayInputStream;
import sun.misc.IOUtils;
import java.security.cert.CertificateException;
import java.util.Hashtable;
import java.io.ObjectInputStream;
import java.security.cert.CertificateEncodingException;
import java.io.IOException;
import java.io.ObjectOutputStream;
import java.util.ArrayList;
import java.security.cert.CertificateFactory;
import java.net.SocketPermission;
import java.security.cert.Certificate;
import java.net.URL;
import java.io.Serializable;

public class CodeSource implements Serializable
{
    private static final long serialVersionUID = 4977541819976013951L;
    private URL location;
    private transient CodeSigner[] signers;
    private transient Certificate[] certs;
    private transient SocketPermission sp;
    private transient CertificateFactory factory;
    
    public CodeSource(URL location, Certificate[] array) {
        this.signers = null;
        this.certs = null;
        this.factory = null;
        this.location = location;
        if (array != null) {
            this.certs = (Certificate[])array.clone();
        }
    }
    
    public CodeSource(URL location, CodeSigner[] array) {
        this.signers = null;
        this.certs = null;
        this.factory = null;
        this.location = location;
        if (array != null) {
            this.signers = (CodeSigner[])array.clone();
        }
    }
    
    @Override
    public int hashCode() {
        if (this.location != null) {
            return this.location.hashCode();
        }
        return 0;
    }
    
    @Override
    public boolean equals(Object o) {
        if (o == this) {
            return true;
        }
        if (!(o instanceof CodeSource)) {
            return false;
        }
        CodeSource codeSource = (CodeSource)o;
        if (this.location == null) {
            if (codeSource.location != null) {
                return false;
            }
        }
        else if (!this.location.equals(codeSource.location)) {
            return false;
        }
        return this.matchCerts(codeSource, true);
    }
    
    public final URL getLocation() {
        return this.location;
    }
    
    public final Certificate[] getCertificates() {
        if (this.certs != null) {
            return (Certificate[])this.certs.clone();
        }
        if (this.signers != null) {
            ArrayList<Object> list = new ArrayList();
            for (int i = 0; i < this.signers.length; ++i) {
                list.addAll(this.signers[i].getSignerCertPath().getCertificates());
            }
            this.certs = (Certificate[])list.toArray(new Certificate[list.size()]);
            return (Certificate[])this.certs.clone();
        }
        return null;
    }
    
    public final CodeSigner[] getCodeSigners() {
        if (this.signers != null) {
            return (CodeSigner[])this.signers.clone();
        }
        if (this.certs != null) {
            this.signers = this.convertCertArrayToSignerArray(this.certs);
            return (CodeSigner[])this.signers.clone();
        }
        return null;
    }
    
    public boolean implies(CodeSource codeSource) {
        return codeSource != null && this.matchCerts(codeSource, false) && this.matchLocation(codeSource);
    }
    
    private boolean matchCerts(CodeSource codeSource, boolean b) {
        if (this.certs == null && this.signers == null) {
            return !b || (codeSource.certs == null && codeSource.signers == null);
        }
        if (this.signers != null && codeSource.signers != null) {
            if (b && this.signers.length != codeSource.signers.length) {
                return false;
            }
            int k = 0;
            while (k < this.signers.length) {
                boolean n2 = false;
                int l = 0;
                while (l < codeSource.signers.length) {
                    if (this.signers[k].equals(codeSource.signers[l])) {
                        n2 = true;
                        break;
                    }
                    ++l;
                }
                if (n2 == 0) {
                    return false;
                }
                ++k;
            }
            return true;
        }
        else if (this.certs != null && codeSource.certs != null) {
            if (b && this.certs.length != codeSource.certs.length) {
                return false;
            }
            int i = 0;
            while (i < this.certs.length) {
                boolean n = false;
                int j = 0;
                while (j < codeSource.certs.length) {
                    if (this.certs[i].equals(codeSource.certs[j])) {
                        n = true;
                        break;
                    }
                    ++j;
                }
                if (n == 0) {
                    return false;
                }
                ++i;
            }
            return true;
        }
        else {
            return false;
        }
    }
    
    private boolean matchLocation(CodeSource codeSource) {
        if (this.location == null) {
            return true;
        }
        if (codeSource == null || codeSource.location == null) {
            return false;
        }
        if (this.location.equals(codeSource.location)) {
            return true;
        }
        if (!this.location.getProtocol().equalsIgnoreCase(codeSource.location.getProtocol())) {
            return false;
        }
        int port = this.location.getPort();
        if (port != -1) {
            int port2 = codeSource.location.getPort();
            if (port != ((port2 != -1) ? port2 : codeSource.location.getDefaultPort())) {
                return false;
            }
        }
        if (this.location.getFile().endsWith("/-")) {
            if (!codeSource.location.getFile().startsWith(this.location.getFile().substring(0, this.location.getFile().length() - 1))) {
                return false;
            }
        }
        else if (this.location.getFile().endsWith("/*")) {
            int lastIndex = codeSource.location.getFile().lastIndexOf(47);
            if (lastIndex == -1) {
                return false;
            }
            if (!codeSource.location.getFile().substring(0, lastIndex + 1).equals(this.location.getFile().substring(0, this.location.getFile().length() - 1))) {
                return false;
            }
        }
        else if (!codeSource.location.getFile().equals(this.location.getFile()) && !codeSource.location.getFile().equals(this.location.getFile() + "/")) {
            return false;
        }
        if (this.location.getRef() != null && !this.location.getRef().equals(codeSource.location.getRef())) {
            return false;
        }
        String host = this.location.getHost();
        String host2 = codeSource.location.getHost();
        if (host != null) {
            if ("".equals(host) || "localhost".equals(host)) {
                if (!"".equals(host2)) {
                    if ("localhost".equals(host2)) {
                        return true;
                    }
                }
                else {
                    return true;
                }
            }
            if (!host.equals(host2)) {
                if (host2 == null) {
                    return false;
                }
                if (this.sp == null) {
                    this.sp = new SocketPermission(host, "resolve");
                }
                if (codeSource.sp == null) {
                    codeSource.sp = new SocketPermission(host2, "resolve");
                }
                if (!this.sp.implies(codeSource.sp)) {
                    return false;
                }
            }
        }
        return true;
    }
    
    @Override
    public String toString() {
        StringBuilder sb = new StringBuilder();
        sb.append("(");
        sb.append(this.location);
        if (this.certs != null && this.certs.length > 0) {
            for (int j = 0; j < this.certs.length; ++j) {
                sb.append(" " + this.certs[j]);
            }
        }
        else if (this.signers != null && this.signers.length > 0) {
            for (int i = 0; i < this.signers.length; ++i) {
                sb.append(" " + this.signers[i]);
            }
        }
        else {
            sb.append(" <no signer certificates>");
        }
        sb.append(")");
        return sb.toString();
    }
    
    private void writeObject(ObjectOutputStream objectOutputStream) throws IOException {
        objectOutputStream.defaultWriteObject();
        if (this.certs == null || this.certs.length == 0) {
            objectOutputStream.writeInt(0);
        }
        else {
            objectOutputStream.writeInt(this.certs.length);
            for (int i = 0; i < this.certs.length; ++i) {
                Certificate certificate = this.certs[i];
                try {
                    objectOutputStream.writeUTF(certificate.getType());
                    byte[] encoded = certificate.getEncoded();
                    objectOutputStream.writeInt(encoded.length);
                    objectOutputStream.write(encoded);
                }
                catch (CertificateEncodingException ex) {
                    throw new IOException(ex.getMessage());
                }
            }
        }
        if (this.signers != null && this.signers.length > 0) {
            objectOutputStream.writeObject(this.signers);
        }
    }
    
    private void readObject(ObjectInputStream objectInputStream) throws IOException, ClassNotFoundException {
        Hashtable hashtable = null;
        Object o = null;
        objectInputStream.defaultReadObject();
        int int1 = objectInputStream.readInt();
        if (int1 > 0) {
            hashtable = new Hashtable(3);
            o = new ArrayList((int1 > 20) ? 20 : int1);
        }
        else if (int1 < 0) {
            throw new IOException("size cannot be negative");
        }
        for (int i = 0; i < int1; ++i) {
            String utf = objectInputStream.readUTF();
            CertificateFactory instance;
            if (hashtable.containsKey(utf)) {
                instance = (CertificateFactory)hashtable.get(utf);
            }
            else {
                try {
                    instance = CertificateFactory.getInstance(utf);
                }
                catch (CertificateException ex) {
                    throw new ClassNotFoundException("Certificate factory for " + utf + " not found");
                }
                hashtable.put(utf, instance);
            }
            ByteArrayInputStream inStream = new ByteArrayInputStream(IOUtils.readExactlyNBytes(objectInputStream, objectInputStream.readInt()));
            try {
                o.add(instance.generateCertificate(inStream));
            }
            catch (CertificateException ex2) {
                throw new IOException(ex2.getMessage());
            }
            inStream.close();
        }
        if (o != null) {
            this.certs = (Certificate[])o.toArray(new Certificate[int1]);
        }
        try {
            this.signers = (CodeSigner[])((CodeSigner[])(CodeSigner[])objectInputStream.readObject()).clone();
        }
        catch (IOException ex3) {}
    }
    
    private CodeSigner[] convertCertArrayToSignerArray(Certificate[] array) {
        if (array == null) {
            return null;
        }
        try {
            if (this.factory == null) {
                this.factory = CertificateFactory.getInstance("X.509");
            }
            int i = 0;
            ArrayList<CodeSigner> list = new ArrayList();
            while (i < array.length) {
                ArrayList certificates = new ArrayList();
                certificates.add(array[i++]);
                for (; i < array.length && array[i] instanceof X509Certificate && ((X509Certificate)array[i]).getBasicConstraints() != -1; ++i) {
                    certificates.add(array[i]);
                }
                list.add(new CodeSigner(this.factory.generateCertPath(certificates), null));
            }
            if (list.isEmpty()) {
                return null;
            }
            return (CodeSigner[])list.toArray(new CodeSigner[list.size()]);
        }
        catch (CertificateException ex) {
            return null;
        }
    }
}
